﻿Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D

Public Class Form1
    Private d As Device = Nothing
    Private WithEvents vb As VertexBuffer = Nothing

    'Next 6 numbers handle the click-drag movement
    Private rotX As Single = 0
    Private rotY As Single = 0
    Private initX As Single = 0
    Private initY As Single = 0
    Private finX As Single = 0
    Private finY As Single = 0

    Private booMovingMouse As Boolean = False

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        InitializeD3D()
    End Sub

    Private Sub InitializeD3D()
        'Create my device and set it up.
        Dim para As New PresentParameters

        para.Windowed = True
        para.SwapEffect = SwapEffect.Discard

        d = New Device(0, DeviceType.Hardware, Me, CreateFlags.HardwareVertexProcessing, para)
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint, True)
        Me.SetStyle(ControlStyles.Opaque, True)

        'Set up and stock the vertex buffer with the vertexes of my cube.
        vb = New VertexBuffer(GetType(CustomVertex.PositionNormalColored), 36, d, Usage.Dynamic And Usage.WriteOnly, CustomVertex.PositionColored.Format, Pool.Default)
        vb.SetData(New Cube(2, 2, 2, Color.SkyBlue).GetVerts, 0, LockFlags.None)
    End Sub

    Private Sub SetupCamera()
        With d
            'Set my field-of-view (perspective) and position-direction.
            .Transform.Projection = Matrix.PerspectiveFovLH(Math.PI / 4, CSng(Me.Width / Me.Height), 1, 100)
            .Transform.View = Matrix.LookAtLH(New Vector3(0, 0, 5.0F), New Vector3(), New Vector3(0, 1, 0))

            .RenderState.Lighting = True

            'My single light
            .Lights(0).Type = LightType.Point
            .Lights(0).Position = New Vector3(0, 0, 10)
            .Lights(0).Ambient = System.Drawing.Color.White
            .Lights(0).Attenuation0 = 0.8F
            .Lights(0).Attenuation1 = 0.02F
            .Lights(0).Range = 10000.0F

            .Lights(0).Enabled = True

        End With
    End Sub

    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        'Process my cube position if the mouse is being click-dragged.  
        If booMovingMouse Then
            rotX = (initX - MousePosition.X) + finX
            rotY = (initY - MousePosition.Y) + finY
        End If

        'Set up my camera and lighting.  Since in most scenes, this changes constantly,
        'it is here and not in the Initialize. 
        SetupCamera()

        'Render the scene.
        With d
            .Clear(ClearFlags.Target, Color.DarkGray, 1, 0)

            .BeginScene()
            .VertexFormat = CustomVertex.PositionNormalColored.Format
            .SetStreamSource(0, vb, 0)

            'Scale the mouse movement by 100 because 1:1 is too sensitive.  Y = negative so 
            'the cube moves the way you'd expect on initial movement.  A true "grab and move 
            'as expected" routine would be a little too complex for this example but all you'd
            'need is a routine to determine the orientation of the world to the camera and select
            'axis of rotation and direction accordingly.
            .Transform.World = Matrix.RotationYawPitchRoll(rotX / 100.0F, -rotY / 100.0F, 0)
            .DrawPrimitives(PrimitiveType.TriangleList, 0, 12)

            .EndScene()
            .Present()
        End With

        Me.Invalidate()
    End Sub

    Private Sub Form1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        booMovingMouse = True
        initX = MousePosition.X
        initY = MousePosition.Y
    End Sub

    Private Sub Form1_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        booMovingMouse = False
        finX += (initX - MousePosition.X)
        finY += (initY - MousePosition.Y)
    End Sub
End Class
